unit Basics;

interface

uses SysUtils;

const
	minList = 1;
  maxList = 5000;
	maxShortList = 15;

// A string_list is an array of strings; .F and .L are the first and last indices.
// Because of the use for multifractions, .S is added for the sign.
type
	string_list = record
		L: integer;
    S: char;
		V: array[minList..maxList] of string;
		end;
	char_list = record
		L: integer;
		V: array[minList..maxList] of char;
		end;
	integer_list = record
		L: integer;
		V: array[0..maxList] of integer;
		end;
  string_multilist = record
		L: integer;
  	V: array[minList..maxShortList] of string_list;
  	P: array[minList..maxShortList] of longint;
  	end;

var	output: text;
		maxNbVertices, maxNbEdges: integer;
    control: char;
    abort, success, counterex: boolean;
   	clock: array[1..32] of double;

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Time management
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure pause;
// startClock(i) sets "clock(i)" to the current time.
procedure startClock(i: integer);
// elapsedTime(i) is the time since clock(i) was started.
function elapsedTime(i: integer): double;
var maxDuration: array[1 .. 20] of double;
procedure askDuration(i: integer);
procedure checkDuration(i: integer; comment: string);

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// General
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure splitInteger2 (n: LongInt; var n1, n2: LongInt);
procedure hRule;

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Strings
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function intToStr(n: integer): string;
function strToInt(u: string): integer;

function nextChar(c: char): char;
function isLetter(c: char): boolean;
function isPosLetter (c: char): boolean;
function isNegLetter (c: char): boolean;
// thePosLetters(u) extracts the list of positive letters in u without repetition
function thePosLetters(u: string): string;
function theNegLetters(u: string): string;
function isPosStr(w: string): boolean;
function isNegStr(w: string): boolean;
// invStr(w) is the "inverse" of w, meaning small and capital
// letters exchanged, and the order reversed.
function invChar(c: char): char;
function invStr(w: string): string;
// mirrorStr(w) is obtained from w by reversing the order of letters.
function mirrorStr(w: string): string;

// subst(u, source, image) replaces in u every letter occurring in "source"
// with the corresponding word in "image", where "image" is the concatenation
// of the images of the various letters separated with "/".
function subst(u, source, image: string): string;

// eraseBlock(u, w) erases all occurrences of u in w;
procedure eraseBlock(u: string; var w: string);
// eraseAllChar(u, w) erases all letters that occur in u in w;
procedure eraseAllChar(u: string; var w: string);
// freeReduce(w) freely reduces w, that is, deletes factors s^{-1}s and ss^{-1}
procedure freeReduce(var w: string);
// freeReduced(w) is TRUE if w is freely reduced.
function freeReduced(w: string): boolean;

function isPrefix(u, v: string): boolean;
// shortLexSmaller(u, v) is TRUE if u is ShortLex strictly smaller than v.
function shortLexSmaller(u, v: string): boolean;

// NE(w) is always nonempty.
function NE(w: string): string;
// compaction(w) is a variant of w in which every letter repetition
// like "aaaaaa" are replaced with "a^6".
function compaction(w: string): string;
// padding(w) is w with various characters added according to "style";
// current options: "$," for inserting commas everywhere, ", " for
// inserting blanks after non-final commas.
function padding(w, style: string): string;

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// String enumeration
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// theString(NN, m) is the NNth string on m letters "a, b..."
// theSignedString(NN, m) is the NNth string on 2m letters "a, b..." and "A, B..."
function theString(NN: longint; m: integer): string;
function theSignedString(NN: longint; m: integer): string;

// Referenced strings are similar to strings but using the letters
// of some reference string and not the first letters "a, b, ...".
// theRefString(NN, wref) is the NNth string in the letters of "wref".
// theELRefString(NN, l, wref) is the NNth string of length l in the letters of "wref".
function theRefString(NN: longint; wref: string): string;
function theELRefString(NN: longint; l: integer; wref: string): string;
// nbRefString(l, wref) is the number of strings of length at most l in the letters of "wref".
// nbELRefString(l, wref) is the number of strings of length l in the letters of "wref".
function nbRefString(l: integer; wref: string): longint;
function nbELRefString(l: integer; wref: string): longint;

// theLetters(w) are the different letters occurring in w (in alphabetical order);
// nbLetters(w) is the nb of different letters occurring in w.
function theLetters(w: string): string;
function nbLetters(w: string): integer;

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Parsing
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// parseAtomicFormula(w, tL, rel, tR) extract from w the three
// components (term, predicate, term) of an atomic formula.
procedure parseAtomicFormula(w: string; var tL, rel, tR: string);

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Lists
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// trivialList(L) is TRUE if all entries of L are empty strings.
function trivialList(A: string_list): boolean;

// strToList(w, sep) is the list obtained from w reading "sep" as a separator;
// listToStr(L, sep) is the converse.
function strToList(w: string; sep: char): string_list;
function listToStr(L: string_list; sep: char): string;
// "style" specifies how entries are separated: "1:", ",", "/", "*", "RC", etc.
procedure writeList(L: string_list; style: string);
procedure writeLnList(L: string_list; style: string);
procedure writeIntegerList(L: integer_list; option: char);
// orderList(L) puts L in increasing order.
procedure orderList(var L: string_list);
procedure reflectList(var L: string_list);

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Multilists
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// splitting(NN, DD) is the list of integers encoded in NN
// relative to the list of lists DD.
function splitting(NN: longint; DD: string_multilist): integer_list;
// multiElement(NN, DD) is the NNth element of the list of lists D.
function multiElement(NN: longint; DD: string_multilist): string_list;

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

implementation

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Time management
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure pause;
begin
  readLn(control);
end;	// pause

// startClock(i) sets "clock(i)" to the current time.
procedure startClock(i: integer);
begin
  clock[i]:= now * 100000;
end;	// startClock

// elapsedTime(i) is the time since clock(i) was started.
function elapsedTime(i: integer): double;
begin
  elapsedTime:= now * 100000 - clock[i];
end;	// elapsedTime

procedure askDuration(i: integer);
begin
	writeLn('Enter the maximal duration of the experiment in seconds: ');
  write('    > ');
	readLn(maxDuration[i]);
end;	// askDuration

procedure checkDuration(i: integer; comment: string);
begin
  if elapsedTime(i) > maxDuration[i] then
    begin
      writeLn;
      writeLn('Computing ', comment, ' is long; continue? Y(es)/N(o)/U(nlimited) ');
      write('    > ');
      readLn(control);
      if control = 'Y' then
      	startClock(i);
      if control = 'U' then
      	maxDuration[i]:= 10000;
      if not(control in ['Y', 'U']) then
      	abort:= TRUE;
    end;
end;	// checkDuration


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// General
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure hRule;
var p: integer;
begin
  for p:= 1 to 80 do
    write('=');
  writeLn;
end;	// hRule

// Bijection of N to NxN by enumerating diagonals of the square;
procedure splitInteger2 (n: LongInt; var n1, n2: LongInt);
var
	p: LongInt;
begin
	if n = 0 then
		begin
			n1 := 0;
			n2 := 0;
		end
	else
		begin
			n := n + 1;
			p := Round(SQRT(2 * n));
			if p * (p + 1) >= 2 * n then
				p := p - 1;
			if p * (p + 1) >= 2 * n then
				p := p - 1;
			n1 := n - p * (p + 1) div 2 - 1;
			n2 := p - n1;
		end;
end;	// splitInteger2


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Strings
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function intToStr(n: integer): string;
var	w: string;
begin
	if n < 0 then
		w:= concat('-', intToStr(-n))
	else if n < 10 then
		w:= Chr(48 + n)
	else
		w:= concat(intToStr(n div 10), Chr(48 + (n mod 10)));
	intToStr:= w;
end;	// intToStr

function strToInt(u: string): integer;
var
	n, p: integer;
begin
	n:= 0;
	for p:= 1 to length(u) do
		n:= 10*n + ord(u[p]) - 48;
	strToInt:= n;
end;	// strToInt

// eraseBlock(u, w) erases in w all occurrences of u;
procedure eraseBlock(u: string; var w: string);
var ww: string;
  p: integer;
begin
  if length(u) = 0 then
   	exit;
	p:= pos(u, w);
  while p > 0 do
    begin
      delete(w, p, length(u));
      p:= pos(u, w);
    end;
end;	// eraseBlock

// eraseAllChar(u, w) erases in w all letters that occur in u;
procedure eraseAllChar(u: string; var w: string);
var p: integer;
begin
 	//writeLn('before w = "', w, '"');
  if length(u) = 0 then
   	exit;
  for p:= 1 to length(u) do
    eraseBlock(u[p], w);
end;	// eraseAllChar

function isNegLetter (c: Char): boolean;
begin
	isNegLetter:= ('A' <= c) and (c <= 'Z');
end;	// isNegLetter

function isPosLetter (c: Char): boolean;
begin
	isPosLetter:= ('a' <= c) and (c  <= 'z');
end;	// isPosLetter

function isLetter(c: char): boolean;
begin
  isLetter:=  isNegLetter(c) or isPosLetter(c);
end;	// isLetter

function nextChar(c: char): char;
begin
	nextChar:= chr(ord(c)+1);
end;	// nextChar

// thePosLetters(u) (resp. theNegLetters(u)) extracts the list of
// positive (resp. negative) letters in u without repetition
function theNegLetters(u: string): string;
var w: string;
  p: integer;
begin
  w:= '';
  for p:= 1 to length(u) do
    if isNegLetter(u[p]) and (pos(u[p], w) = 0) then
      w:= concat(w, u[p]);
  theNegLetters:= w;
end;	// theNegLetters

// thePosLetters(u) extracts the list of negative letters
// in u without repetition
function thePosLetters(u: string): string;
var w: string;
  p: integer;
begin
  w:= '';
  for p:= 1 to length(u) do
    if isPosLetter(u[p]) and (pos(u[p], w) = 0) then
      w:= concat(w, u[p]);
  thePosLetters:= w;
end;	// thePosLetters

function invChar (c: Char): Char;
begin
	if isPosLetter(c) then
		invChar:= Chr(Ord(c) - 32)
	else if isNegLetter(c) then
		invChar:= Chr(Ord(c) + 32)
  else
    invChar:= c;
end;	// invChar

// invStr(w) is the "inverse" of w, meaning small and capital
// letters exchanged, and the order reversed.
function invStr(w: string): string;
var	u: string;
	p: integer;
begin
	u := '';
	for p := 1 to length(w) do
		u := concat(invChar(w[p]), u);
	invStr := u;
end;	// invStr

// mirrorStr(w) is obtained from w by reversing the order of letters.
function mirrorStr(w: string): string;
var	u: string;
  p: integer;
begin
	u:= '';
	for p:= 1 to length(w) do
		u:= concat(w[p], u);
	mirrorStr:= u;
end;	// mirrorStr

function isNegStr(w: string): boolean;
var p: integer;
begin
  isNegStr:= FALSE;
  for p:= 1 to length(w) do
    if not(isNegLetter(w[p])) then
      exit;
  isNegStr:= TRUE;
end;	// isNegStr

function isPosStr(w: string): boolean;
var p: integer;
begin
  isPosStr:= FALSE;
  for p:= 1 to length(w) do
    if not(isPosLetter(w[p])) then
      exit;
  isPosStr:= TRUE;
end;	// isPosStr

// freeReduced(w) is TRUE if w is freely reduced.
function freeReduced(w: string): boolean;
var p: integer;
begin
	freeReduced:= FALSE;
  for p:= 1 to length(w) - 1 do
    if w[p + 1] = invChar(w[p]) then
      exit;
	freeReduced:= TRUE;
end;	// freeReduced

function isPrefix(u, v: string): boolean;
var p: integer;
begin
  isPrefix:= FALSE;
  if length(v) < length(u) then
    exit;
  for p:= 1 to length(u) do
    if u[p] <> v[p] then
      exit;
  isPrefix:= TRUE;
end;

// shortLexSmaller(u, v) is TRUE if u is ShortLex strictly smaller than v.
function shortLexSmaller(u, v: string): boolean;
begin
	shortLexSmaller:= TRUE;
  if length(u) < length(v) then
		exit;
	if (length(u) > length(v)) or (u = '') then
		begin
      shortLexSmaller:= FALSE;
      exit;
    end;
	if u[1] < v[1] then
		exit;
	if u[1] > v[1] then
		begin
      shortLexSmaller:= FALSE;
      exit;
    end;
	u:= copy(u, 2, 255);
	v:= copy(v, 2, 255);
  shortLexSmaller:= shortLexSmaller(u, v);
end;	// shortLexSmaller

// subst(u, source, image) replaces in u every letter occurring in "source"
// with the corresponding word in "image", where "image" is the concatenation
// of the images of the various letters separated with "/".
function subst(u, source, image: string): string;
var i, p: integer;
  uu: string;
  S: string_list;
begin
  S:= strToList(image, '/');
  uu:= '';
  for i:= 1 to length(u) do
    begin
      p:= pos(u[i], source);
      if p > 0 then
      	uu:= concat(uu, S.V[p])
    	else
      	uu:= concat(uu, u[i]);
    end;
  subst:= uu;
end;	// subst

// freeReduce(w) freely reduces w, that is, deletes factors s^{-1}s and ss^{-1}
procedure freeReduce(var w: string);
var u: string;
    p: integer;
begin
  repeat
 		if w = '' then
      exit;
   	u:= w;
    for p:= length(w)-1 downto 1 do
			if w[p+1] = invChar(w[p]) then
        delete(w, p, 2);
  until (u = w) or (length(w) = 0);
end;	// freeReduce

// NE(w) is always nonempty;
function NE(w: string): string;
begin
  if w <> '' then
    NE:= w
  else
    NE:= '1';
end;

// compaction(w) is a variant of w in which every letter repetition
// like "aaaaaa" are replaced with "a^6".
function compaction(w: string): string;
var i, k: integer;
	v: string;
	c: char;
begin
	if w = '' then
		v:= 'empty'
	else if length(w) = 255 then
		v:= 'too long'
	else
		begin
			v:= '';
			c:= w[1];
			k:= 1;
			for i:= 2 to length(w) do
				if w[i] = w[i-1] then
 					k:= k+1
				else
				 	begin
					 	if k = 1 then
					 		v:= concat(v, c)
					 	else
					 		v:= concat(v, c, '^', intToStr(k));
					 	c:= w[i];
					 	k:= 1;
 					end;
				if k = 1 then
					v:= concat(v, c)
				else
					v:= concat(v, c, '^', intToStr(k));
			end;
	compaction:= v;
end;	// compaction

// padding(w) is w with various characters added according to "style";
// current options: "$," for inserting commas everywhere, ", " for
// inserting blanks after non-final commas.
function padding(w, style: string): string;
var ww: string;
  p: integer;
begin
  ww:= '';
  for p:= 1 to length(w) do
    case style of
      '$,': if p < length(w) then
              ww:= concat(ww, w[p], ',')
            else
              ww:= concat(ww, w[p]);
      ', ': begin
              ww:= concat(ww, w[p]);
              if w[p] = ',' then
              	if p < length(w) then
                	ww:= concat(ww, ' ')
                else
                  delete(ww, length(ww), 1);
        		end;
      otherwise
      	ww:= w;
      end;
	padding:= ww;
end;	// padding


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// % String enumeration
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// theString(NN, m) is the NNth string on m letters "a, b..."
// theSignedString(NN, m) is the NNth string on 2m letters "a, b..." and "A, B..."
function theString(NN: longint; m: integer): string;
var p: integer;
		u: string;
begin
	u:= '';
	while NN > 0 do
		begin
			p:= NN mod m;
			if p > 0 then
        u:= concat(Chr(96 + p), u)
      else
        u:= concat(Chr(96 + m), u);
			NN:= (NN-1) div m;
		end;
	theString:= u;
end;	// theString

function theSignedString(NN: longint; m: integer): string;
var p, mm: integer;
		u: string;
begin
	mm:= 2 * m;
  u:= '';
	while NN > 0 do
		begin
			p:= NN mod mm;
			if p = 0 then
        u:= concat(Chr(64 + m), u)
      else if p < m then
				u:= concat(Chr(96 + p), u)
			else if p = m then
				u:= concat(Chr(96 + m), u)
			else
				u:= concat(Chr(64 + p - m), u);
			NN:= (NN-1) div mm;
		end;
	theSignedString:= u;
end;	// theSignedString

// theRefString(NN, wref) is the NNth string in the letters of "wref".
function theRefString(NN: longint; wref: string): string;
var n, p: integer;
		v: string;
begin
	n:= length(wref);
  v:= theString(NN, n);
  for p:= 1 to length(v) do
    v[p]:= wref[ord(v[p]) - 96];
	theRefString:= v;
end;	// theRefString

// theELRefString(NN, l, wref) is the NNth string of length l in the letters of "wref".
function theELRefString(NN: longint; l: integer; wref: string): string;
var n, p, i: integer;
		u: string;
begin
  n:= length(wref);
	u:= '';
	for p:= 1 to l do
    begin
			i:= NN mod n;
      if i > 0 then
      	u:= concat(wref[i], u)
      else
      	u:= concat(wref[n], u);
      NN:= (NN + n - 1) div n;
    end;
	theELRefString:= u;
end;	// theELRefString

// nbRefString(l, wref) is the number of strings of length at most l in the letters of "wref".
function nbRefString(l: integer; wref: string): longint;
var	k: integer;
    n, NN: longint;
begin
  n:= length(wref);
  if n = 1 then
    nbRefString:= l + 1
  else
    begin
      NN:= 1;
      for k:= 1 to l + 1 do
    		NN:= NN * n;
  		nbRefString:= (NN - 1) div (n - 1) - 1;
    end;
end;	// nbRefString

// nbELRefString(l, wref) is the number of strings of length l in the letters of "wref".
function nbELRefString(l: integer; wref: string): longint;
var	k: integer;
    NN: longint;
begin
  NN:= 1;
  for k:= 1 to l do
    NN:= NN * length(wref);
  nbELRefString:= NN;
end;	// nbELRefString

// theLetters(w) are the different letters occurring in w (in alphabetical order);
// nbLetters(w) is the nb of different letters occurring in w.
function theLetters(w: string): string;
var c: char;
  v: string;
begin
	v:= '';
  for c:= 'a' to 'z' do
    if pos(c, w) > 0 then
      v:= concat(v, c);
	theLetters:= v;
end;	// theLetters

// nbLetters(w) is the nb of different letters occurring in w
function nbLetters(w: string): integer;
begin
	nbLetters:= length(theLetters(w));
end;	// nbLetters


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Parsing
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// parseAtomicFormula(w, tL, rel, tR) extract from w the three
// components of an atomic formula.
procedure parseAtomicFormula(w: string; var tL, rel, tR: string);
var p: integer;
  position: char;
begin
  tL:= '';
  rel:= '';
  tR:= '';
 	position:= 'L';
  for p:= 1 to length(w) do
    case position of
      'L':
        begin
          if isLetter(w[p]) then
    				tL:= concat(tL, w[p]);
          if w[p] = '=' then
    				begin
              rel:= concat(rel, w[p]);
              position:= 'R';
            end;
          if w[p] = '\' then
    				begin
              rel:= concat(rel, w[p]);
              position:= 'M';
            end;
        end;
      'M':
        if w[p] <> ' ' then
  				rel:= concat(rel, w[p])
        else
					position:= 'R';
      'R':
      	if isLetter(w[p]) then
  				tR:= concat(tR, w[p]);
    end;
  //writeLn('w = ', w, ', tL = ', tL, ', rel = ', rel, ', tR = ', tR);
end;	// parseAtomicFormula


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Lists
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// trivialList(L) is TRUE if all entries of L are empty strings.
function trivialList(A: string_list): boolean;
var i: integer;
begin
  trivialList:= FALSE;
  for i:= minList to A.L do
    if A.V[i] <> '' then
      exit;
  trivialList:= TRUE;
end;	// trivialList

// strToList(w, sep) is the list obtained from w reading "sep" as a separator.
function strToList(w: string; sep: char): string_list;
var	p, i: integer;
  u: string;
  L: string_list;
begin
  i:= minList;
  u:= '';
  for p:= 1 to length(w) do
    if w[p] = sep then
    	begin
				L.V[i]:= u;
        i:= i + 1;
        u:= '';
      end
  	else if w[p] <> ' ' then
    	u:= concat(u, w[p]);
	L.V[i]:= u;
  L.L:= i;
  strToList:= L;
end;	// strToList

// "sep" specifies the separator between entries.
function listToStr(L: string_list; sep: char): string;
var	i: integer;
  w: string;
begin
	if L.L = -1 then
    begin
      listToStr:= '$';
      exit;
    end;
  w:= '';
  for i:= minList to L.L - 1 do
		w:= concat(w, L.V[i], sep);
  w:= concat(w, L.V[L.L]);
  listToStr:= w;
end;	// listToStr

procedure writeList(L: string_list; style: string);
var	p: integer;
begin
	for p:= minList to L.L do
		begin
			if pos('1:', style) > 0 then
        write(p, ': ');
			if L.V[p] = '' then
				write('1')
			else
				write(L.V[p]);
      if pos(',', style) > 0 then
        if p < L.L then
         	write(', ');
      if pos(';', style) > 0 then
        write('; ');
      if pos('*', style) > 0 then
        if p < L.L then
         	write('*');
      if pos('/', style) > 0 then
        if p < L.L then
          write('/');
      if pos('RC', style) > 0 then
          writeLn;
		end;
end;	// writeList

procedure writeLnList(L: string_list; style: string);
begin
	writeList(L, style);
	writeLn;
end;	// writeLnList

procedure writeIntegerList(L: integer_list; option: char);
var	p: integer;
begin
	for p:= minList to L.L do
		begin
			if p > minList then
        case option of
          ',': write(', ');
          '/': write('/');
          '*': write('*');
          '-': write('');
        otherwise
        	writeLn;
        end;
      write(L.V[p]);
		end;
end;	// writeIntegerList

// orderList(L) puts L in increasing order.
procedure orderList(var L: string_list);
var p: integer;
		w: string;
begin
	if L.L < minList then
    exit;
	repeat
    p:= minList;
  	while (p + 1 <= L.L) and (L.V[p] < L.V[p+1]) do
  		p:= p + 1;
		if p + 1 <= L.L then
// We found a pair that is not in the right order
    	begin
      	w:= L.V[p];
      	L.V[p]:= L.V[p + 1];
      	L.V[p + 1]:= w;
    	end;
  until p + 1 > L.L;
end;	// orderList

procedure reflectList(var L: string_list);
var	i: integer;
		w: string;
begin
	for i:= minList to (minList + L.L) div 2 do
		begin
			w:= L.V[i];
			L.V[i]:= L.V[minList + L.L - i];
			L.V[minList + L.L - i]:= w;
		end;
end;	// reflectList


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Multilists
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// splitting(NN, DD) is the list of integers encoded in NN
// relative to the list of lists DD.
function splitting(NN: longint; DD: string_multilist): integer_list;
var i, p: integer;
  LI: integer_list;
begin
  LI.L:= DD.L;
  for i:= minList to DD.L do
    begin
      p:= (NN - 1) div DD.P[i + 1];
      NN:= NN - p * DD.P[i + 1];
      LI.V[i]:= p + 1;
    end;
  splitting:= LI;
end;	// splitting

// multiElement(NN, DD) is the NNth element of the list of lists D.
function multiElement(NN: longint; DD: string_multilist): string_list;
var i: integer;
    LI: integer_list;
    L: string_list;
begin
  LI:= splitting(NN, DD);
  L.L:= DD.L;
  for i:= minList to DD.L do
    L.V[i]:= DD.V[i].V[LI.V[i]];
  multiElement:= L;
end;	// multiElement

end.

