unit Experiments;

{$mode objfpc}{$H+}

interface

uses SysUtils, Basics, Reversing, Garside, Presentation, Fractions, Reductions,
Graphs, Global;

var sampleSize, cumulDepth, cumulLength, nbExp: longint;
  averDepth, averLength: real;

procedure ConjBRandom;
procedure ConjBAllExpOne;
procedure ConjBAllExpAll;
procedure ConjCRandom;
procedure ConjCunifRandom;
procedure proportionMoreThanOneIrred;

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


implementation

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Conjecture B for random fractions
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure conjBRandom;
var A: string_list;
  w: string;
begin
	hRule;
  if isleft3Ore and isRight3Ore then
    begin
      writeLn('The 3-Ore condition holds, hence Conjecture B must be true');
      hRule;
      exit;
    end;
  writeLn('This procedure tests Conjecture B: ');
	writeLn('  "If a is a unital multifraction, then redtame(a) = 1".');
	writeLn('If this is true, then "a reduces to 1" is true a fortiori,');
  writeLn('so Conjecture A is tested at the same time.');
  writeLn;
	enterUnitalFracSpecif(w);
  if repeatFrac then
    askDuration(1)
  else
    maxDuration[1] := 0;
  writeLn;
  writeLn('(computing)');
	writeLn;
  cumulDepth:= 0;
  cumulLength:= 0;
  nbExp:= 0;
 	startClock(1);
  repeat
  	nbExp:= nbExp + 1;
 		startClock(2);
    A:= theFrac(w);
    if elapsedTime(2) > 1 then
    	writeLn('generating ', frac(A), ' required ', elapsedTime(2):2:2, ' secs.');
    cumulDepth:= cumulDepth + A.L;
    cumulLength:= cumulLength + lengthFrac(A);
    A:= redTame(A);
    if not(trivialFrac(A)) then
      begin
        writeLn('COUNTER-EXAMPLE FOUND!');
        writeLn(frac(A), ' is unital, irreducible, and redTame(a) is not trivial.');
        pause;
      end;
  until elapsedTime(1) >= maxDuration[1];
  if nbExp >= 1 then
    begin
      averDepth:= cumulDepth/nbExp;
  		averLength:= cumulLength/nbExp;
    end;
  writeLn('Conclusion: Conjecture B was positively tested for ');
  write('    ', nbExp, ' random unital multifractions ');
  case genOption of
  	'$': write('generated by trivial pairs and relations; ');
  	'#': write('generated by lcm-expansion of a cross; ');
  end;
  writeLn;
  writeLn(' average length ', averLength:2:2, '; average depth ', averDepth:2:2, '; total computation time ', elapsedTime(1):2:2, ' secs.');
	hRule;
end;	// conjBRandom


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Conjecture B for all lcm-expansions of one fraction
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure ConjBAllExpOne;
var NN, nbDiv: longint;
  i, m: integer;
  A, B, D: string_list;
  DD: string_multilist;
  w: string;
begin
  hRule;
  if isleft3Ore and isRight3Ore then
    begin
      writeLn('The 3-Ore condition holds, hence Conjecture B must be true');
      hRule;
      exit;
    end;
	writeLn('This procedure tests Conjecture B: ');
	writeLn('  "If a is a unital multifraction, then redtame(a) = 1".');
	writeLn('If this is true, then "a reduces to 1" is true a fortiori,');
  writeLn('so Conjecture A is tested at the same time.');
  writeLn;
  writeLn('Here one tests all lcm-expansions of one multifraction.');
  enterCrossSpecif(w);
  if repeatFrac then
    askDuration(1)
  else
    maxDuration[1] := 0;
  writeLn;
  A:= theBoundary(w);
 	startClock(1);
  writeLn('Exploring all lcm-expansions of ', frac(A));
	DD:= allMultiLeftDiv(A);
  nbDiv:= DD.V[1].L;
  write('There are ', nbDiv);
  for i:= 2 to A.L do
    begin
      m:= DD.V[i].L;
      write('*', m);
      nbDiv:= nbDiv * m;
    end;
  write(' divisor combinations,');
  writeLn(' hence at most ', nbDiv, ' lcm-expansions.');
  writeLn;
  writeLn('(computing)');
  writeLn;
  cumulDepth:= 0;
  cumulLength:= 0;
  nbExp:= 0;
  NN:= 1;
  repeat
    D:= multiElement(NN, DD);
    B:= lcmExp(A, D);
    if B.L >= 0 then
      begin
        cumulDepth:= cumulDepth + B.L;
        cumulLength:= cumulLength + lengthFrac(B);
        nbExp:= nbExp + 1;
        if not(trivialFrac(redTame(B))) then
          begin
            writeLn('COUNTER-EXAMPLE FOUND!');
            writeLn(frac(B), ' is unital, irreducible, and redTame(b) is not trivial.');
            pause;
          end;
      end;
    NN:= NN + 1;
  until (NN > DD.P[1]) or (elapsedTime(1) > maxDuration[1]);
  if nbExp >= 1 then
    begin
      averDepth:= cumulDepth/nbExp;
  		averLength:= cumulLength/nbExp;
    end;
  writeLn('Conclusion: Conjecture B was positively tested for ');
  if NN > DD.P[1] then
    write('all ')
  else
  	write('the first ');
  writeLn(nbExp, ' lcm-expansions of ', frac(A));
	writeLn(' average length ', averLength:2:2, '; average depth ', averDepth:2:2, '; total computation time ', elapsedTime(1):2:2, ' secs.');
  hRule;
end;


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Conjecture B for all lcm-expansions of all central crosses
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure ConjBAllExpAll;
label 1;
var NN, nbDiv, totalNb: longint;
  n, k, l, i, m: integer;
  A, B, D, X: string_list;
  choice: char;
  DD: string_multilist;
  NN1, NN2, NN3, NN4, NN5, NN6: longint;
begin
	hRule;
  if isleft3Ore and isRight3Ore then
    begin
      writeLn('The 3-Ore condition holds, hence Conjecture B must be true');
      hRule;
      exit;
    end;
  writeLn('This procedure tests Conjecture B: ');
	writeLn('  "If a is a unital multifraction, then redtame(a) = 1".');
	writeLn('If this is true, then "a reduces to 1" is true a fortiori,');
  writeLn('so Conjecture A is tested at the same time.');
  writeLn;
  writeLn('Here one tests all lcm-expansions of all (short) central crosses.');
  write('Enter the maximal length length of the rays of the central cross (1 to 3): ');
  readLn(l);
  write('Enter the width of multifraction (4 or 6): ');
  askDuration(1);
  writeLn;
  writeLn('(computing)');
  writeLn;
  if n < 6 then
    n:= 4
  else
    n:= 6;
  totalNb:= 1;
  for i:= 1 to n do
    totalNb:= totalNb * nbRefString(l, theGen);
  writeLn('total nb ', totalNb);
 	startClock(1);
  nbExp:= 1;
  cumulDepth:= 0;
  cumulLength:= 0;
  nbExp:= 1;
	if n = 4 then
    for NN1:= 0 to nbRefString(l, theGen) do
    	for NN2:= 0 to nbRefString(l, theGen) do
      	for NN3:= 0 to nbRefString(l, theGen) do
        	for NN4:= 0 to nbRefString(l, theGen) do
            begin
              X.V[1]:= theRefString(NN1, theGen);
              X.V[2]:= theRefString(NN2, theGen);
              X.V[3]:= theRefString(NN3, theGen);
              X.V[4]:= theRefString(NN4, theGen);
              X.L:= 4;
//              writeLn(frac(X));
              A:= boundary(X);
            	DD:= allMultiLeftDiv(A);
              nbDiv:= DD.V[1].L;
              for NN:= 1 to DD.P[1] do
                begin
                  D:= multiElement(NN, DD);
                  B:= lcmExp(A, D);
                  if B.L > 0 then
                    begin
                      cumulDepth:= cumulDepth + B.L;
                  		cumulLength:= cumulLength + lengthFrac(B);
                    end;
                  if (B.L >= 0) and not(trivialFrac(redTame(B))) then
                    begin
                      writeLn('COUNTER-EXAMPLE FOUND!');
                      writeLn(frac(B), ' is unital, irreducible, and redTame(b) is not trivial.');
                      pause;
                    end;
                end;
              writeLn('OK for all lcm-expansions stemming from ', listToStr(X, '#'));
      				nbExp:= nbExp + 1;
              if elapsedTime(1) > maxDuration[1] then
                goto 1;
            end;
	if n = 6 then
    for NN1:= 0 to nbRefString(l, theGen) do
    	for NN2:= 0 to nbRefString(l, theGen) do
      	for NN3:= 0 to nbRefString(l, theGen) do
        	for NN4:= 0 to nbRefString(l, theGen) do
          	for NN5:= 0 to nbRefString(l, theGen) do
            	for NN6:= 0 to nbRefString(l, theGen) do
                begin
                  X.V[1]:= theRefString(NN1, theGen);
                  X.V[2]:= theRefString(NN2, theGen);
                  X.V[3]:= theRefString(NN3, theGen);
                  X.V[4]:= theRefString(NN4, theGen);
                  X.V[5]:= theRefString(NN5, theGen);
                  X.V[6]:= theRefString(NN6, theGen);
                  X.L:= 6;
//                  writeLn(frac(X));
                  A:= boundary(X);
                	DD:= allMultiLeftDiv(A);
                  nbDiv:= DD.V[1].L;
                  for NN:= 1 to DD.P[1] do
                    begin
                      D:= multiElement(NN, DD);
                      B:= lcmExp(A, D);
                      if B.L > 0 then
                        begin
                          cumulDepth:= cumulDepth + B.L;
                      		cumulLength:= cumulLength + lengthFrac(B);
                        end;
                      if (B.L >= 0) and not(trivialFrac(redTame(B))) then
                        begin
                          writeLn('COUNTER-EXAMPLE FOUND!');
                          writeLn(frac(B), ' is unital, irreducible, and redTame(b) is not trivial.');
                          pause;
                        end;
                    end;
                  writeLn('OK for all lcm-expansions stemming from ', listToStr(X, '#'));
          				nbExp:= nbExp + 1;
                  if elapsedTime(1) > maxDuration[1] then
                    goto 1;
                end;
1:averDepth:= cumulDepth/NN;
  averLength:= cumulLength/NN;
  nbExp:= nbExp - 1;
  writeLn('Conclusion: Conjecture B was positively tested for');
  write(' all lcm-expansions of ');
  if nbExp >= totalNb then
    write('all ')
  else
  	write('the first ');
  write(nbExp, ' multifractions ');
  if nbExp < totalNb then
  	write('out of a total of ', totalNb);
  writeLn;
  writeLn(' that have depth ', n, ' and admit a central cross with rays of length <= ', l, '.');
	writeLn(' total computation time ', elapsedTime(1):2:2, ' secs.');
  hRule;
end;	// ConjBAllExpAll


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Conjecture C for random fractions
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure conjCRandom;
var k, p, q: integer;
  strat, LIL, LIR, A: string_list;
  w, aa, bb, cc: string;
  found, good, acceptable: boolean;
begin
	hRule;
  if isleft3Ore and isRight3Ore then
    begin
      writeLn('The 3-Ore condition holds, hence Conjecture C must be true');
      hRule;
      exit;
    end;
  writeLn('This procedure tests Conjecture C: ');
	writeLn('  "Any two right-reducts of a admit a common left-reduct".');
	writeLn('Because testing completely is long, the test here is partial:');
	writeLn('one looks for the right-irreducible right-reducts of a');
	writeLn('corresponding to the usual 4 reduction strategies, and one checks');
	writeLn('that they all left-reduce to at least one of the left-irreducible');
	writeLn('obtained by the same usual 4 reduction strategies.');
  writeLn;
	enterFracSpecif(w);
  if repeatFrac then
    askDuration(1)
  else
    maxDuration[1] := 0;
  writeLn;
  writeLn('(computing)');
  writeLn;
  cumulDepth:= 0;
  cumulLength:= 0;
  startClock(1);
  nbExp:= 0;
  strat.V[1]:= 'ALba';
  strat.V[2]:= 'ALbz';
  strat.V[3]:= 'ALta';
  strat.V[4]:= 'ALtz';
  strat.V[5]:= 'ARba';
  strat.V[6]:= 'ARbz';
  strat.V[7]:= 'ARta';
  strat.V[8]:= 'ARtz';
  repeat
    nbExp:= nbExp + 1;
    write(nbExp, ': ');
    A:= theFrac(w);
    cumulDepth:= cumulDepth + A.L;
    cumulLength:= cumulLength + lengthFrac(A);
    aa:= frac(A);
    LIL.L:= 0;
		for k:= 1 to 4 do
    	begin
        bb:= aa;
        stratReduce(bb, strat.V[k]);
        appendFrac(bb, LIL);
    	end;
    LIR.L:= 0;
		for k:= 5 to 8 do
    	begin
        bb:= aa;
        stratReduce(bb, strat.V[k]);
        appendFrac(bb, LIR);
    	end;
// Now LIL is our list of left-reducibles, and LIR is the list of right-irreducibles.
		acceptable:= FALSE;
    for p:= 1 to LIL.L do
      begin
        bb:= LIL.V[p];
        good:= TRUE;
        for q:= 1 to LIR.L do
          begin
            found:= FALSE;
            k:= 1;
            repeat
							cc:= LIR.V[q];
              stratReduce(cc, strat.V[k]);
              if equivpFrac(cc, bb) then
                begin
                  accessible[q, p]:= TRUE;
                  found:= TRUE;
//                  writeLn(q, ' left-reduces to ', p);
                end;
              k:= k + 1;
            until (k > 4) or found;
            if not(found) then
              good:= FALSE;
          end;
        acceptable:= acceptable or good;
      	if not(good) then
          write('some right-reduct does not reduce to LR#', p, '; ')
        else
        	write('all right-reducts reduce to LR#', p, '; ');
      end;
    writeLn;
    if not(acceptable) then
      begin
        writeLn('FOUND COUNTER-EXAMPLE: For a = ', aa, ' it might be that ');
        writeLn('some right-reduct of a does not left-reduce to any left-irred(a).');
      end;
  until elapsedTime(1) > maxDuration[1];
  writeLn;
  if nbExp >= 1 then
    begin
      averDepth:= cumulDepth/nbExp;
  		averLength:= cumulLength/nbExp;
    end;
  writeLn('Conclusion: The weak form of Conjecture C was positively tested for ');
  writeLn(' ', nbExp, ' random multifractions:');
	writeLn(' average length ', averLength:2:2, '; average depth ', averDepth:2:2, '; total computation time ', elapsedTime(1):2:2, ' secs.');
	hRule;
end;	// conjCRandom


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Conjecture C_unif for random fractions
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure ConjCunifRandom;
var A: string_list;
	w, aa: string;
  LRL, LRR, LR: frac_list;
  LEL, LER, LE: edge_list;
begin
	hRule;
  if isleft3Ore and isRight3Ore then
    begin
      writeLn('The 3-Ore condition holds, hence Conjecture C must be true');
      hRule;
      exit;
    end;
  writeLn('This procedure tests Conjecture C_unif: ');
	writeLn('    "For every multifraction A, there exists a left-reduct nabla(A)');
	writeLn('     of A such that every right-reduct of a left-reduces to nabla(A)."');
  writeLn;
	enterFracSpecif(w);
  if repeatFrac then
    askDuration(1)
  else
    maxDuration[1]:= 0;
  writeLn;
  nbExp:= 0;
  cumulDepth:= 0;
  cumulLength:= 0;
  startClock(1);
  repeat
    A:= theFrac(w);
    displayCrossConf(A, 'W', maxDuration[1]);
    if not(abort) then
      begin
        nbExp:= nbExp + 1;
        cumulDepth:= cumulDepth + A.L;
        cumulLength:= cumulLength + lengthFrac(A);
      end;
  until elapsedTime(1) > maxDuration[1];
  if nbExp >= 1 then
    begin
      averDepth:= cumulDepth/nbExp;
  		averLength:= cumulLength/nbExp;
      writeLn;
      writeLn('Conclusion: Conjecture C_unif was positively tested for ', nbExp, ' multifractions; ');
    	writeLn('average length ', averLength:2:2, '; average depth ', averDepth:2:2, '; total computation time ', elapsedTime(1):2:2, ' secs.');
    end
  else
  	begin
      writeLn('Conclusion: Conjecture C_unif could not be tested (too long). ');
    end;
  hRule;
end;	// ConjCunifRandom


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Proportion of more than one irreducible reduct
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// Computes the ratio of multifractions with more than one
// irreducible reduct.
procedure proportionMoreThanOneIrred;
var NN, NNb: longint;
  aa: string;
  m, l, n, k, p, q: integer;
  r, r1, v: real;
  Names: integer_list;
  A: string_list;
begin
  writeLn('This procedure computes the proportion of multifractions');
  writeLn('with more than one irreducible reduct, for various AT monoids.');
  write('Enter the maximal size of the samples: ');
  readLn(sampleSize);
  askDuration(1);
  Names.L:= 6;
  Names.V[1]:= 1;
  Names.V[2]:= 2;
  Names.V[3]:= 3;
  Names.V[4]:= 6;
  Names.V[5]:= 8;
  Names.V[6]:= 9;
  for q:= 1 to Names.L do
    begin
      writeLn;
      writeLn('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
      p:= Names.V[q];
      installPresentation(recordedPres[p], 'w');
      writeLn(recordedName[p]);
    	for k:= 1 to 5 do
        begin
          cumulDepth:= 0;
          l:= 8 * k;
          n:= 3 + k;
          NNb:= 0;
          NN:= 0;
 					startClock(1);
          repeat
          	NN:= NN + 1;
           	//aa:= randomFixedDepthFrac(l, n);
            A:= randomFrac(l);
            cumulDepth:= cumulDepth + depth(aa);
   					startClock(2);
            A:= redTameIt(A);
            if elapsedTime(2) > 2 then
            	writeLn('    one example of depth ', A.L, ' where computing ∂tame requires ', elapsedTime(2):2:2, ' secs.');
            if not(reduced(A)) then
              NNb:= NNb + 1;
            if NN = sampleSize div 2 then
              r1:= NNb/NN;
          until (NN >= sampleSize) or (elapsedTime(1) >= maxDuration[1]);
          r:= NNb/NN;
          v:= r - r1;
          if v < 0 then
            v:= -v;
          averDepth:= cumulDepth/NN;
          writeLn(NN:5, ' random fractions of length ', l, ' in ', elapsedTime(1):2:2, ' sec.');
          writeLn('  average depth = ', averDepth:2:2, '; ratio = ', r:5:3, ' ± ', v:5:3);
        end;
    end;
	writeLn;
  writeLn('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
end;	// proportionMoreThanOneIrred

end.

