unit Garside;

interface

uses SysUtils, Basics, Reversing;

var atomList, leftPrimList, leftPrimExt, rightPrimList, rightPrimExt: string_list;
  leftSimpleList, leftGarExt, rightSimpleList, rightGarExt: string_list;
// symPrim is TRUE if the left and right primitive elements coincide.
  noetherian, stronglyNoetherian, leftPrimInstalled, rightPrimInstalled, symPrim: boolean;
  isLeft2Ore, isLeft3Ore, isLeft4Ore, isRight2Ore, isRight3Ore, isRight4Ore, GarMon, is3Ore: boolean;
  CELeft2Ore, CERight2Ore, CELeft3Ore, CERight3Ore, CELeft4Ore, CERight4Ore: string;
  Ore, GarInstalled, Delta, righPrimNotLeftPrim: string;

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Greedy normal form
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// head(u, S) is the right-lcm of the elements of S that left-divide u;
// whenever S is closed under right-lcm, this is the greatest element
// of S left-dividing u, hence the head of u relative to S.
function head(u: string; S: string_list): string;
function greedy(u, v: string): boolean;
procedure findHead;
// greedyNF(u, S) is the greedy normal form of the positive string u
// with respect to the (Garside) family S
function greedyNF(u: string; S: string_list): string;
function symGreedyNF(w: string; S: string_list): string;
function NF(u: string): string;
function symNF(u: string): string;

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Primitive elements
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function closure(L: string_list; oper, option: string): string_list;
function extremalElements(L: string_list; side: char): string_list;
procedure installAtomList(option: string);
procedure installPrimLists(option: string);
procedure displayPrimElts(option: string);

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Simple elements
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure installSimpleLists(option: string);
procedure displaySimpleElts(option: string);

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Ore conditions
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure checkOreConditions(option: string);
procedure displayOre(option: string);


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

implementation

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Greedy normal form
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// head(u, S) is the right-lcm of the elements of S that left-divide u;
// whenever S is closed under right-lcm, this is the greatest element
// of S left-dividing u, hence the head of u relative to S.
function head(u: string; S: string_list): string;
var	i: integer;
	v: string;
begin
	v:= '';
	for i:= minList to S.L do
		if leftDiv(S.V[i], u) then
			v:= rightLcm(v, S.V[i]);
	head:= v;
end;	// head

function greedy(u, v: string): boolean;
begin
	greedy:= (head(concat(u, v), leftSimpleList) = u);
end;	// greedy

procedure findHead;
var	u: string;
begin
	writeLn('Enter one word.');
	readLn(u);
	writeLn('The head of ', u, ' is ', head(u, leftSimpleList), '.');
end;	// findHead

// greedyNF(u, S) is the greedy normal form of the positive string u
// with respect to the (Garside) family S
function greedyNF(u: string; S: string_list): string;
var uu, h: string;
begin
	uu:= '';
  repeat
    h:= head(u, S);
    u:= rightComp(h, u);
    if uu = '' then
      uu:= h
    else
      uu:= concat(uu, '|', h);
  until u = '';
  greedyNF:= uu;
end;	// greedyNF

// Attention! In a monoid not satisfying the 4-property, this
// symmetric normal form might not be unique.
function symGreedyNF(w: string; S: string_list): string;
var u, v, uu, vv: string;
begin
  symGreedyNF:= '$';
  leftRev(w, u, v);
  if u <> '$' then
    begin
      uu:= rightRes(u, v);
      vv:= rightRes(v, u);
      if uu = '' then
      	symGreedyNF:= greedyNF(vv, S)
      else if vv = '' then
      	symGreedyNF:= invStr(greedyNF(uu, S))
      else
    		symGreedyNF:= concat(invStr(greedyNF(uu, S)), '|', greedyNF(vv, S));
    end;
end;	// symGreedyNF

function NF(u: string): string;
begin
  if pos('R', GarInstalled) > 0 then
    NF:= greedyNF(u, leftSimpleList)
  else
    NF:= u;
  if u = '' then
    NF:= '1';
end;	// NF

function symNF(u: string): string;
begin
  if pos('R', GarInstalled) > 0 then
    symNF:= symGreedyNF(u, leftSimpleList)
  else
    symNF:= u;
  if u = '' then
    symNF:= '1';
end;	// symNF


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Primitive elements
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function closure(L: string_list; oper, option: string): string_list;
var S: string_list;
    p, q: integer;
    u, v: string;
    stop: double;
begin
	if L.L = 0 then
    begin
      S.L:= 0;
      closure:= S;
      exit;
    end;
  S:= L;
  append('', S);
	p:= minList + 1;
  abort:= FALSE;
  startClock(1);
  maxDuration[1]:= 30;
  repeat
    for q:= minList to p - 1 do
      case oper of
      'RC':
        begin
          rightRevv(S.V[p], S.V[q], u, v);
          if u <> '$' then
            begin
              append(u, S);
              append(v, S);
            end;
        end;
      'RM':
        begin
          rightRevv(S.V[p], S.V[q], u, v);
          if u <> '$' then
            append(concat(S.V[q], u), S)
          else
            if CERight3Ore = '' then
              CERight3Ore:= concat(S.V[p], '/', S.V[q]);
        end;
      'LC':
        begin
          leftRevv(S.V[p], S.V[q], u, v);
          if u <> '$' then
            begin
              append(u, S);
              append(v, S);
            end;
        end;
      'LM':
        begin
          leftRevv(S.V[p], S.V[q], u, v);
          if u <> '$' then
            append(concat(u, S.V[q]), S)
          else
            if CELeft2Ore = '' then
              CELeft2Ore:= concat(S.V[p], '/', S.V[q]);
        end;
      end;
    p:= p + 1;
    checkDuration(1, 'the closure');
  until (p > S.L) or abort;
	if abort then
		S.L:= 0;
  closure:= S;
end;	// closure

function extremal(p: integer; L: string_list; side: char): boolean;
var q, i: integer;
begin
	if side = 'R' then
    i:= 1
  else
    i:= 0;
  extremal:= FALSE;
	for q:= minList to L.L do
    if (q <> p) and flexDiv(i, L.V[p], L.V[q]) then
      exit;
  extremal:= TRUE;
end;	// extremal

function extremalElements(L: string_list; side: char): string_list;
var	E: string_list;
  p: integer;
begin
  E.L:= 0;
  for p:= minList to L.L do
    if extremal(p, L, side) then
      append(L.V[p], E);
	extremalElements:= E;
end;	// extremalElements

procedure installAtomList(option: string);
var i: integer;
	L: string_list;
begin
  for i:= 1 to nbGen do
    L.V[i]:= theGen[i];
  L.L:= nbGen;
  atomList:= L;
end;	// installAtomList

procedure installPrimLists(option: string);
var L: string_list;
  k: integer;
begin
// Initialisation
  if pos('W', option) > 0 then
    write('Determining primitive elements: ');
// Determining right-primitive elements
	L:= closure(atomList, 'RC', option);
  rightPrimInstalled:= (L.L > 0);
  rightPrimList:= L;
 	if pos('W', option) > 0 then
  	if rightPrimInstalled then
    	write('done on the right; ')
    else
			write('aborted on the right; ');
// Determining left-primitive elements
  L:= closure(atomList, 'LC', option);
  leftPrimInstalled:= (L.L > 0);
  leftPrimList:= L;
 	if pos('W', option) > 0 then
  	if leftPrimInstalled then
    	write('done on the left. ')
    else
			write('aborted on the left. ');
// Comparing left and right primitive elements
	symPrim:= FALSE;
	righPrimNotLeftPrim:= '$';
  if rightPrimInstalled and leftPrimInstalled then
		begin
      symPrim:= equivListPlus(rightPrimList, leftPrimList, k);
      if (rightPrimList.L = leftPrimList.L) and not(symPrim) then
    	  righPrimNotLeftPrim:= rightPrimList.V[k];
    end;
  if pos('W', option) > 0 then
    writeLn;
end;	// installPrimLists

procedure displayPrimElts(option: string);
begin
  if (pos('W', option) = 0) and (pos('w', option) = 0) then
    exit;
  if GarMon then
    exit;
  if symPrim then
    begin
    	write('The left and right primitive elements coincide; there are ', rightPrimList.L);
    	write(' of them');
      if pos('W', option) > 0 then
        begin
          writeLn(', namely ');
          write('    ');
    			writeList(rightPrimList, ',');
          writeLn('. ');
        end;
    end;
  if rightPrimInstalled and not(symPrim) then
    begin
    	write('There are ', rightPrimList.L, ' right-primitive elements');
      if pos('W', option) > 0 then
        begin
          write(', namely ');
    			writeList(rightPrimList, ',');
          writeLn('. ');
        end
      else
      	write('. ');
    end;
  if leftPrimInstalled and not(symPrim) then
    begin
    	write('There are ', leftPrimList.L, ' left-primitive elements');
      if pos('W', option) > 0 then
        begin
          write(', namely ');
    			writeList(leftPrimList, ',');
          writeLn('. ');
        end
      else
      	write('. ');
    end;
  if righPrimNotLeftPrim <> '$' then
    writeLn('Example of a right-primitive that is not left-primitive: ', righPrimNotLeftPrim, '.');
  if not(rightPrimInstalled) then
    writeLn('Right-primitive elements could not be determined.');
  if not(leftPrimInstalled) then
   	writeLn('Left-primitive elements could not be determined.');
end;	// displayPrimElts


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Simple elements
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure installSimpleLists(option: string);
var L, LE: string_list;
  DeltaR, DeltaL: string;
begin
// Initialisation
  if pos('W', option) > 0 then
    write('Determining simple elements: ');
	GarInstalled:= '';
// Determining the smallest right-Garside family
  L:= closure(rightPrimList, 'RM', option);
  rightSimpleList:= L;
 	LE:= extremalElements(rightSimpleList, 'R');
  rightGarExt:= LE;
  if L.L > 0 then
    GarInstalled:= concat(GarInstalled, 'R');
  if pos('W', option) > 0 then
    if pos('R', GarInstalled) > 0 then
      write('done on the right; ')
    else
      write('aborted on the right; ');
// Determining the smallest left-Garside family
  L:= closure(leftPrimList, 'LM', option);
  leftSimpleList:= L;
 	LE:= extremalElements(leftSimpleList, 'L');
  leftGarExt:= LE;
  if L.L > 0 then
    GarInstalled:= concat(GarInstalled, 'L');
  if pos('W', option) > 0 then
    if L.L > 0 then
      write('done on the left; ')
    else
      write('aborted on the left; ');
// Recognizing a Garside monoid
	GarMon:= FALSE;
  Delta:= '$';
  if (leftGarExt.L = 1) and (rightGarExt.L = 1) then
    begin
      DeltaR:= rightGarExt.V[1];
      DeltaL:= leftGarExt.V[1];
      if equiv(DeltaR, DeltaL) then
        begin
          Delta:= DeltaR;
          GarMon:= TRUE;
        end;
    end;
// Conclusion
  if pos('W', option) > 0 then
    writeLn;
end;	// installSimpleLists

procedure displaySimpleElts(option: string);
begin
  if pos('W', option) = 0 then
    exit;
  if GarMon then
    begin
      writeLn('The monoid is a Garside monoid, with ');
      write('    ∆ = ', Delta, ', and ');
    	writeLn(rightSimpleList.L, ' divisors of ∆. ');
    	write('There are ', rightPrimList.L, ' right-primitive elements and ');
    	writeLn(leftPrimList.L, ' left-primitive elements. ');
			exit;
    end;
  if pos('R', GarInstalled) > 0 then
    begin
      write('There are ', rightSimpleList.L, ' right-simple elements, namely the right-divisors of ');
      if rightGarExt.L = 1 then
        writeLn('the element ')
      else
      	writeLn('the ', rightGarExt.L, ' elements ');
      write('   ');
  		writeList(rightGarExt, ', ');
    	writeLn('. ');
      //writeList(rightSimpleList, ', ');writeLn;
    end
  else
    write('Right-simple elements could not be determined (aborted). ');
  if pos('L', GarInstalled) > 0 then
    begin
      write('There are ', leftSimpleList.L, ' left-simple elements, namely the left-divisors of ');
      if leftGarExt.L = 1 then
        writeLn('the element ')
      else
      	writeLn('the ', leftGarExt.L, ' elements ');
      write('   ');
  		writeList(leftGarExt, ', ');
    	writeLn('. ');
    end
  else
    writeLn('Left-simple elements could not be determined (aborted). ');
  if not(noetherian) then
  	writeLn('But nothing is certain as the monoid is not known to be noetherian.');
  writeLn;
end;	// displaySimpleElts


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Ore conditions
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure checkOreConditions(option: string);
label R2, R3, R4, L2, L3, L4;
var p, q, r, s: integer;
  L: string_list;
begin
  if pos('W', option) > 0 then
    begin
      write('Checking Ore conditions: ');
      if not(rightPrimInstalled) then
        write('aborted on the right; ');
      if not(leftPrimInstalled) then
        writeLn('aborted on the left. ');
    end;
  Ore:= '';
// right Ore condition
R2:if not(rightPrimInstalled) then
    goto L2;
  L:= rightPrimList;
// right 2-Ore condition
	isRight2Ore:= FALSE;
	for p:= 1 to L.L do
    for q:= p + 1 to L.L do
      if rightLcm(L.V[p], L.V[q]) = '$' then
      	begin
        	CERight2Ore:= concat(L.V[p], '/', L.V[q]);
        	goto R3;
      	end;
 	isRight2Ore:= TRUE;
  if pos('W', option) > 0 then
    write('done on the right; ');
  goto L2;
// right 3-Ore condition
R3:isRight3Ore:= FALSE;
	for p:= 1 to L.L do
    for q:= p + 1 to L.L do
      for r:= q + 1 to L.L do
      	if rightLcm(L.V[p], L.V[q]) <> '$' then
        	if rightLcm(L.V[q], L.V[r]) <> '$' then
          	if rightLcm(L.V[p], L.V[r]) <> '$' then
     					if rightLcm3(L.V[p], L.V[q], L.V[r]) = '$' then
              	begin
                	CERight3Ore:= concat(L.V[p], '/', L.V[q], '/', L.V[r]);
                	goto R4;
              	end;
 	isRight3Ore:= TRUE;
  if pos('W', option) > 0 then
    write('done on the right; ');
  goto L2;
// right 3-Ore condition
R4:isRight4Ore:= FALSE;
	for p:= 1 to L.L do
    for q:= p + 1 to L.L do
      for r:= q + 1 to L.L do
        for s:= r + 1 to L.L do
      		if rightLcm3(L.V[p], L.V[q], L.V[r]) <> '$' then
        		if rightLcm3(L.V[p], L.V[q], L.V[s]) <> '$' then
          		if rightLcm3(L.V[p], L.V[r], L.V[s]) <> '$' then
            		if rightLcm3(L.V[q], L.V[r], L.V[s]) <> '$' then
              		if rightLcm4(L.V[p], L.V[q], L.V[r], L.V[s]) = '$' then
                    begin
                    	CERight4Ore:= concat(L.V[p], '/', L.V[q], '/', L.V[r], '/', L.V[s]);
                    	goto L2;
                    end;
  isRight4Ore:= TRUE;
	if pos('W', option) > 0 then
  	write('done on the right; ');
// left Ore condition
L2:if not(rightPrimInstalled) then
    exit;
  L:= leftPrimList;
// left 2-Ore condition
	isLeft2Ore:= FALSE;
	for p:= 1 to L.L do
    for q:= p + 1 to L.L do
      if leftLcm(L.V[p], L.V[q]) = '$' then
      	begin
        	CELeft2Ore:= concat(L.V[p], '/', L.V[q]);
        	goto L3;
      	end;
 	isLeft2Ore:= TRUE;
  if pos('W', option) > 0 then
    writeLn('done on the left. ');
  exit;
// left 3-Ore condition
L3:	isLeft3Ore:= FALSE;
	for p:= 1 to L.L do
  	for q:= p + 1 to L.L do
    	for r:= q + 1 to L.L do
    		if leftLcm(L.V[p], L.V[q]) <> '$' then
      		if leftLcm(L.V[q], L.V[r]) <> '$' then
        		if leftLcm(L.V[p], L.V[r]) <> '$' then
   						if leftLcm3(L.V[p], L.V[q], L.V[r]) = '$' then
            		begin
              		CELeft3Ore:= concat(L.V[p], '/', L.V[q], '/', L.V[r]);
              		goto L4;
            		end;
  isLeft3Ore:= TRUE;
  is3Ore:= isRight3Ore;
  if pos('W', option) > 0 then
    writeLn('done on the left. ');
	exit;
// left 4-Ore condition
L4:isLeft4Ore:= FALSE;
	for p:= 1 to L.L do
    for q:= p + 1 to L.L do
      for r:= q + 1 to L.L do
        for s:= r + 1 to L.L do
      		if leftLcm3(L.V[p], L.V[q], L.V[r]) <> '$' then
        		if leftLcm3(L.V[p], L.V[q], L.V[s]) <> '$' then
          		if leftLcm3(L.V[p], L.V[r], L.V[s]) <> '$' then
            		if leftLcm3(L.V[q], L.V[r], L.V[s]) <> '$' then
              		if leftLcm4(L.V[p], L.V[q], L.V[r], L.V[s]) = '$' then
                    begin
                    	CELeft4Ore:= concat(L.V[p], '/', L.V[q], '/', L.V[r], '/', L.V[s]);
                    	exit;
                    end;
  isLeft4Ore:= TRUE;
	if pos('W', option) > 0 then
  	writeLn('done on the left. ');
end;	// checkOreConditions

procedure displayOre(option: string);
label 1;
var L: string_list;
begin
  if pos('W', option) = 0 then
    exit;
  if GarMon then
    begin
	    write('The 2-Ore condition holds on both sides: any two elements admit a common ');
      writeLn('right-multiple and a common left-multiple.');
      exit;
    end;
// right Ore conditions
  write('Ore condition (recall: the right n-Ore condition holds if any n elements which admit n-1 by n-1 ');
	writeLn('a common right-multiple admit a global common right-multiple): ');
  if not(rightPrimInstalled) then
    writeLn('unknown, right-primitive elements could not be determined.')
  else if isRight2Ore then
    writeLn('The right 2-Ore condition holds. ')
  else
  	begin
    	L:= strToList(CERight2Ore, '/');
    	writeLn('The right 2-Ore condition fails for ', L.V[1], ' and ', L.V[2], '. ');
      if isRight3Ore then
        begin
        	writeLn('The right 3-Ore condition holds. ');
        end
      else
      	begin
          L:= strToList(CERight3Ore, '/');
          writeLn('The right 3-Ore condition fails for ', L.V[1], ', ', L.V[2], ', and ', L.V[3], '. ');
          if isRight4Ore then
            begin
              writeLn('The right 4-Ore condition holds. ');
            end
          else
          	begin
              L:= strToList(CERight4Ore, '/');
              writeLn('The right 4-Ore condition fails for ', L.V[1], ', ', L.V[2], ', ', L.V[3], ', and ', L.V[4], '. ');
            end;
        end;
    end;
// left Ore conditions
  if not(leftPrimInstalled) then
    writeLn('Left Ore condition: unknown, left-primitive elements could not be determined.')
  else if isLeft2Ore then
    writeLn('The left 2-Ore condition holds. ')
  else
  	begin
    	L:= strToList(CELeft2Ore, '/');
    	writeLn('The left 2-Ore condition fails for ', L.V[1], ' and ', L.V[2], '. ');
      if isLeft3Ore then
        begin
        	writeLn('The left 3-Ore condition holds. ');
        end
      else
      	begin
          L:= strToList(CELeft3Ore, '/');
          writeLn('The left 3-Ore condition fails for ', L.V[1], ', ', L.V[2], ', and ', L.V[3], '. ');
          if isLeft4Ore then
            begin
              writeLn('The left 4-Ore condition holds. ');
            end
          else
          	begin
              L:= strToList(CELeft4Ore, '/');
              writeLn('The left 4-Ore condition fails for ', L.V[1], ', ', L.V[2], ', ', L.V[3], ', and ', L.V[4], '. ');
            end;
        end;
    end;
  if isRight2Ore then
    begin
    	write('As a consequence, every element of the group is represented by a unique irreducible right fraction, ');
      writeLn('and multifraction reduction is convergent. ');
    end
  else if isLeft2Ore then
    begin
    	write('As a consequence, every element of the group is represented by a unique irreducible left fraction, ');
      writeLn('and multifraction reduction is convergent. ');
    end
  else if is3Ore then
    begin
    	write('As a consequence, every element of the group is represented by a unique irreducible multifraction, ');
      writeLn('and multifraction reduction is convergent. ');
    end
  else
    writeLn('As a consequence, multifraction reduction is not convergent.');
end;	// displayOre


end.


