program MoKa;

{$mode objfpc}{$H+}

// This is the main program for MoKa: it contains the interface
// procedures, initialization, menu, etc. In principle, all
// technical procedures should be in the various units, unless
// they are still in progress.

// The directive $H+ forces all strings to be ANSIstrings;
// removing it produces an error; it might be good to introduce
// and use a type "shortString" for short sequences.

uses SysUtils, Basics, Reversing, Garside, Presentation, Fractions, Reductions,
Graphs, Global, Experiments;

var menu: char;

procedure testAssumption;
var aa, w: string;
  A: string_list;
    l, n: integer;
    NN, maxNN: longint;
    submenu: char;
begin
  maxNN:= 10000000;
  writeLn;
  writeLn('Choose the way the test is done:');
	writeLn('  One fraction .......................................... O ');
	writeLn('  Random fractions with pause ........................... r ');
	writeLn('  Random fractions, no pause ............................ R ');
	writeLn('  All fractions ......................................... A ');
	write('    > ');
  readLn(submenu);
  case submenu of
    'r', 'R':
      begin
      	writeLn('Enter length, possibly depth:');
      	writeLn('  - type l for random fractions of length l,');
      	writeLn('  - type l/n for random fractions of length l and depth n,');
      	writeLn('  - type l$ for random trivial fractions of length l,');
      	writeLn('  - type l/n$ for random trivial random of length <=l and depth n.');
      	write('    > ');
      	readLn(w);
    	end;
		end;
  writeLn;
  NN:= 0;
  repeat
    NN:= NN + 1;
    case submenu of
      'O': 	begin
        			enterFrac(A);
        			NN:= maxNN + 1;
        		end;
      'r', 'R':
				A:= theFrac(w);
      'A':
        A:= list(fracForm(theSignedString(NN, nbGen)));
      end;
    counterex:= FALSE;
//  	test(aa, NN);
    if counterex then
      begin
				writeLn(NN, ': found counterexample for ', aa);
        pause;
      end;
    if submenu = 'r' then
      pause;
    if NN mod 100000 = 0 then
      begin
        writeLn(NN, ' successful tries; last try ', aa);
        pause;
      end;
  until (control = 'Q') or (NN >= maxNN);
end;	// testAssumption

procedure experiment1;
var w: string;
  A: string_list;
begin
// writeLn('Vacant for further experiments');
  writeLn('Enter specif');
  write('    > ');
  readLn(w);
  repeatFrac:= (w[1] in ['0'..'9']);
  repeat
    A:= theFrac(w);
    writeLn('A = ', frac(A));
  until not(unitalFrac(A));
end;	// experiment1

procedure experiment2;
var p, q, l: integer;
  A, B, C: string_list;
  w: string;
  nbNonTrivial: longint;
begin
//  writeLn('Vacant for further experiments');
  writeLn('Investigation of minimal unital multifractions');
  enterUnitalFracSpecif(w);
  startClock(1);
  nbExp:= 0;
  nbNonTrivial:= 0;
  repeat
    nbExp:= nbExp + 1;
    A:= theFrac(w);
 		A:= derDivPlus(A);
    if not(trivialFrac(A)) then
      nbNonTrivial:= nbNonTrivial + 1;
    l:= lengthFrac(A);
    for p:= 1 to l - 1 do
    	for q:= p + 1 to l do
    		begin
        	B:= extract(A, p, q);
        	if unitalFrac(B) then
        		begin
              C:= derDivPlus(B);
              if not(trivialFrac(C)) then
              	writeLn(nbExp, ': the ', p, '-', q, '-extract, ', frac(B), ' is unital');
            end;
        end;
  until elapsedTime(1) > maxDuration[1];
  writeLn(nbExp, ' total tries; ', nbNonTrivial, ' not trivial after division.');
  writeLn('No nontrivial minimal unital multifraction has been found.');
end;	// experiment2


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Control
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

procedure initialize;
var log: char;
begin
	maxNbRev:= 1000;
	maxLengthRev:= 1000;
  maxNbVertices:= maxList;
  maxNbEdges:= maxList;
  infty:= maxList;
	writeLn();
	hRule;
  writeLn('   MoKa (monoid calculus), version 1.3 (Nov.15, 2016)');
  writeLn('   Patrick DEHORNOY, LMNO, Universite de Caen, 14032 Caen, France');
	writeLn('   patrick.dehornoy@unicaen.fr');
	writeLn('   www.math.unicaen.fr/~dehornoy/');
	hRule;
  writeLn;
  chooseAccessPath;
  setRecordedPres;
  installPresentation(ArtinRel('A333'), 'PGW');
end;	// initialize

procedure help;
begin
	hRule;
	writeLn('Explanations about this program');
  writeLn;
	writeLn('This program implements various algorithms for working in a monoid defined');
  writeLn('by a complemented presentation, typically an Artin-Tits monoid, and in its');
  writeLn('enveloping group. The elements of the monoid are handled via words in the');
  writeLn('alphabet of the specified presentation; the elements of the enveloping group');
  writeLn('are specified by signed words, or by multifractions, i.e., finite sequences');
  writeLn('of elements of the monoid. The main tools are subword reversing for the elements' );
  writeLn('of the monoid, and multifraction reduction for those of the group.');
  writeLn;
	writeLn('Complemented presentations');
  writeLn;
	writeLn('A monoid presentation, i.e., a list of relations u = v where u, v are (positive)');
	writeLn('word in the reference alphabet S is called "right-complemented" if, for each ');
	writeLn('pair of generators s,t, there exists at most one relation of the form s...=t...');
	writeLn('in the presentation. If (S,R) is a right-complemented presentation and moreover');
	writeLn('it is noetherian and satisfies a combinatorial condition called the right');
	writeLn('cube condition, then it is known that two (positive) S-words u, v represent the');
	writeLn('same element of the monoid iff the signed S-word u^{-1}v right-reverses to the');
	writeLn('empty word, where right-reversing is the signed word transformation consisting');
	writeLn('in deleting factors s^{-1}s and replacing factors s^{-1}t with vu^{-1} if sv=tu');
	writeLn('is a relation of the presentation. This is the basic procedure used in MoKa.');
	writeLn('The approach is known to work for all Artin-Tits presentations. The user chooses');
	writeLn('whatever complemented presentation he wishes, but the program works only if the');
	writeLn('noetherianity and cube conditions can be checked.');
  writeLn;
  writeLn('Reference:');
  writeLn(' [1] Complete positive group presentations, J. Algebra 268 (2003) 156-197');
	writeLn;
	writeLn('Multifraction reduction');
  writeLn;
	writeLn('If M is a monoid, the elements of the enveloping group G of M can be ');
	writeLn('represented by multifractions on M, namely finite sequences of elements of M ');
	writeLn('where the successive entries are thought as having alternating signs. ');
	writeLn('Reduction is a rewrite system, whose rules transform a multifraction ');
	writeLn('into an equivalent one, i.e., one that represents the same element of G.');
	writeLn('So, a multifraction rewriting into a trivial multifraction 1/.../1 certainly');
	writeLn('represents 1 in G. Here one is interested in the converse implication, namely');
	writeLn('whether every multifraction representing 1 in G necessarily rewrites into a ');
  writeLn('trivial multifraction. This is true for every Artin-Tits monoid of type FC [1], ');
  writeLn('and conjecturally for every Artin-Tits monoid [2]. The program proposes to try' );
  writeLn('reduction on various multifractions and to test the above conjecture.');
  writeLn;
  writeLn('References:');
  writeLn(' [1] Multifraction reduction I: The 3-Ore case and Artin-Tits groups of type FC');
  writeLn('                                                            arXiv:1606.08991');
  writeLn(' [2] Multifraction reduction II: Conjectures for Artin-Tits groups');
  writeLn('                                                            arXiv:1606.08995');
  writeLn(' [3] (with F.WEHRUNG) Multifraction reduction III: The case of interval monoids');
  writeLn('                                                            arXiv:1606.09018');
  writeLn;
  writeLn('The initial default monoid is the Artin-Tits monoid of type A2tilde.');
	hRule;
end;	// help

procedure resetParameters;
begin
	writeLn('The current value of the escape length in reversing is ', maxLengthRev);
	write('Enter the new value: ');
	readLn(maxLengthRev);
	writeLn('The current value of the escape number of steps in reversing is ', maxNbRev);
	write('Enter the new value (maximal possible value: 15388): ');
	readLn(maxNbRev);
	writeLn('The maximal size of a string_list is currently ', maxList);
end;	// resetParameters

procedure quit;
begin
	writeLn('bye.');
end;	// quit

procedure askmenu;
begin
	writeLn;
  hRule;
  writeLn('MAIN MENU Choose a procedure:');
  writeLn('  Help');
	writeLn('    Explanations about the program and references ....................... H ');
	writeLn('    Reset the base parameters ........................................... r ');
  writeLn('  Reference monoid');
	writeLn('    Show the current reference monoid ................................... p ');
	writeLn('    Enter a new presentation ............................................ P ');
	writeLn('  Work with elements of the monoid');
	writeLn('    Compare two elements, normal form, lcm, gcd, etc., .................. L ');
	writeLn('    List the divisors of an element ...... .............................. D ');
	writeLn('    List the elements satisfying a formula .............................. M ');
	writeLn('  Work with elements of the enveloping group');
	writeLn('    Reduce one multifraction ............................................ O ');
	writeLn('    Compare two multifractions .......................................... T ');
	writeLn('    List the elements satisfying a formula .............................. G ');
  writeLn('  Recorded experiments:');
  writeLn('    Test Conjecture B for one or random multifractions .................. B ');
  writeLn('    Test Conjecture B for all lcm-expansions of one multifraction ....... E ');
  writeLn('    Test Conjecture B for all lcm-expansions of all multifractions ...... A ');
  writeLn('    Test Conjecture C for random multifractions ......................... C ');
  writeLn('    Test Conjecture C_unif for one or random multifractions ............. U ');
	writeLn('  Further procedures');
  writeLn('    Study proportion of more than one irreducible reduct ................ I ');
//	writeLn('    Analyze two fractions ............................................... Y ');
//	writeLn('    Experiment 1 ........................................................ x ');
//	writeLn('    Experiment 2 ........................................................ X ');
	writeLn('  Quit .................................................................. Q ');
	write('     > ');
	readLn(menu);
	hRule;
  writeLn;
end;	// askmenu

{$R *.res}

begin
	initialize;
	repeat
		askmenu;
		if menu <> 'Q' then
			case menu of
        '3': embedding;
        '4': gcdPreserve;
        'A': ConjBAllExpAll;
        'B': ConjBRandom;
        'C': ConjCRandom;
				'D': listDivisors;
        'E': ConjBAllExpOne;
				'G': solveFormula('G');
				'H': help;
        'I': ProportionMoreThanOneIrred;
				'L': treatTwoWords;
				'M': solveFormula('M');
				'O': treatOneFrac;
				'p': showPresentation('W');
				'P': enterPresentation;
				'r': resetParameters;
				'T': compareTwoFrac;
        'U': ConjCunifRandom;
				'x': experiment1;
				'X': experiment2;
				'Y': analyzeTwoFractions;
			end;
	until menu = 'Q';
	quit;
end.
